//
//
// C++ Implementation: $MODULE$
//
// Description: 
//
//
// Author: Bosselut Cyril <bosselut@b1project.com>, (C) 2003
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "tags.h"
#define _NB_GENRES ( sizeof(id3_genres)/sizeof(id3_genres[0]) )

const int Tags::NB_GENRES = _NB_GENRES;

Tags::Tags( const QString & fn ){
	file = fn;
	haveCover = false;
	KMimeMagicResult *result = KMimeMagic::self()->findFileType(file);
	mime = result->mimeType();
}


Tags::~Tags(){
}

void Tags::setInfos(const QString & tr, const QString & ar, const QString & ti, const QString & al, const QString & ge, const QString & ye, const QString & ct, const QString & co){
	track = tr;
	artist = ar;
	stitle = ti;
	album = al;
	genre = ge;
	year = ye;
	comment = ct;
	if(co != ""){
		ncover = co;
		haveCover = true;
	}
}

bool Tags::isMP3(){
	if(mime == "audio/x-mp3"){
		return true;
	}
	return false;
}

void Tags::ReadId3(){
	myTag = new ID3_Tag();
	myTag->Link(file, ID3TT_ALL);
	const Mp3_Headerinfo * mp3header = myTag->GetMp3HeaderInfo();
	rate = mp3header->bitrate;
	int chmode = mp3header->channelmode;
	if(chmode == 0 || chmode == 1 || chmode == 2){
		channel = 2;
	}
	else if(chmode == 3 || chmode == 4){
		channel = 1;
	}
	else{
		channel = 0;
	}
	int tmpfreq = mp3header->frequency;
	freq = ((QString::number(tmpfreq).toDouble())/1000);
	kdDebug("Tags") << "Getting tags from MP3 file" << endl;
	stitle = upFirst(ID3_GetTitle(myTag));
	artist = upFirst(ID3_GetArtist(myTag));
	album = upFirst(ID3_GetAlbum(myTag));
	year = ID3_GetYear(myTag);
	comment = ID3_GetComment(myTag);
	track = QString::number(ID3_GetTrackNum(myTag));
	int idGenre = ID3_GetGenreNum(myTag);
	if(idGenre <= NB_GENRES){
		genre = id3_genres[idGenre];
	}
	if(stitle != ""){
		title = artist + " - " + stitle + " - " + album;
	}
	ID3_Frame* myFrame = myTag->Find(ID3FID_PICTURE);// Check if id3 have PICTURE field
	if (NULL != myFrame){
		kdDebug("Tags") << "id3 have picture " << endl;
		ID3_Field* myField = myFrame->GetField(ID3FN_DATA);
		const uchar* data = myField->GetRawBinary();
		size_t size = myField->Size();
		cover.loadFromData (data, size);
		haveCover = true;
	}
	myTag = 0;
}

void Tags::WriteId3(){
	kdDebug("Tags") << "Adding id3 tags to file " << file << endl;
	myTag = new ID3_Tag();
	myTag->Link(file, ID3TT_ALL);
	ID3_AddTitle(myTag, stitle, true);
	ID3_AddAlbum(myTag, album, true);
	ID3_AddArtist(myTag, artist, true);
	ID3_RemoveComments(myTag);
	ID3_AddComment(myTag, comment, true);
	QStringList * table = new QStringList();
	for(int i = 0; i < NB_GENRES; i++)
	{
		table->append(id3_genres[i]);
	}
	int idGenre(table->findIndex(genre));
	ID3_AddGenre(myTag, idGenre, true);
	ID3_AddYear(myTag, year, true);
	ID3_AddTrack(myTag, track.toInt() , '0', true);
	if(haveCover){
		ID3_Frame* myFrame = myTag->Find(ID3FID_PICTURE);
		KMimeMagicResult *result = KMimeMagic::self()->findFileType(ncover);
		if (NULL != myFrame){
			myFrame->Field(ID3FN_DATA).FromFile(ncover);
			myFrame->Field(ID3FN_MIMETYPE).Set(result->mimeType());
		}
		else{
			myFrame = new ID3_Frame(ID3FID_PICTURE);
			if (NULL != myFrame){
				myFrame->Field(ID3FN_DATA).FromFile(ncover);
				myFrame->Field(ID3FN_MIMETYPE).Set(result->mimeType());
				myTag->AttachFrame(myFrame);
			}
		}
	}
	myTag->Update();
	myTag = 0;
}

bool Tags::isOGG(){
	if(mime == "application/x-ogg" || mime == "audio/vorbis"){
		return true;
	}
	return false;
}

void Tags::ReadVorbis(){
	OggVorbis_File  o;
	FILE *f = fopen( QFile::encodeName(file), "rb");
	int t = ov_open(f, &o, NULL, 0);
	if(t == 0){
		// this part comes from KTagit http://ktagit.sourceforge.net/
		char **ptr = ov_comment(&o, -1)->user_comments;
		kdDebug("Tags") << "Getting tags from OGG file" << endl;
		for(; *ptr; ptr++){
			QString tmp(QString::fromUtf8(*ptr));
			kdDebug("Tags")<< tmp << endl;
			if(tmp.find('=')){
				QString tag (tmp.section('=', 0, 0).lower());
				QString value (tmp.section('=', 1, 1));
				if(!tag.compare("title")){
					stitle = upFirst(value);
				}
				if(!tag.compare("artist")){
					artist = upFirst(value);
				}
				if(!tag.compare("album")){
					album = upFirst(value);
				}
				if(tag.contains("track")){
					track = value;
				}
				if(tag.contains("date")){
					year = value;
				}
				if(tag.contains("genre")){
					genre = value;
				}
				if(tag.contains("comment")){
					comment = value;
				}
			}
		}
		//---//
		if(stitle != ""){
			title = artist + " - " + stitle + " - " + album;
			rate = ov_bitrate(&o, -1);
		}
		ov_clear(&o);
	}
	else{
		kdError("Tags") << "ReadVorbis() " << t << endl;
		fclose(f);
	}
}

void Tags::WriteVorbis(){
	kdDebug("Tags") << "Adding OGG tags to file " << file << endl;
	KFileMetaInfo* info = new KFileMetaInfo ( file );
	if(!info->containsGroup("Comment")){
		info->addGroup("Comment");
		kdDebug("Tags") << "Adding Comment group" << endl;
	}
	KFileMetaInfoGroup group = info->group("Comment");
	if(!group.contains("Title")){
		group.addItem("Title");
	}
	if(!group.contains("Artist")){
		group.addItem("Artist");
	}
	if(!group.contains("Album")){
		group.addItem("Album");
	}
	if(!group.contains("Genre")){
		group.addItem("Genre");
	}
	if(!group.contains("Track")){
		group.addItem("Track");
	}
	if(!group.contains("Date")){
		group.addItem("Date");
	}
	if(!group.contains("Year")){
		group.addItem("Year");
	}
	if(!group.contains("Tracknumber")){
		group.addItem("Tracknumber");
	}
	if(!group.contains("Comment")){
		group.addItem("Comment");
	}
	if(!group.contains("Description")){
		group.addItem("Description");
	}
	info->item("Title").setValue(stitle);
	info->item("Artist").setValue(artist);
	info->item("Album").setValue(album);
	info->item("Genre").setValue(genre);
	info->item("Comment").setValue(comment);
	info->item("Description").setValue(comment);
	info->item("Track").setValue(track);
	info->item("Tracknumber").setValue(track);
	info->item("Date").setValue(year);
	info->item("Year").setValue(year);
	info->applyChanges();
}

bool Tags::darkcover(){
	kdDebug("Tags") << "Searching cover on darktown.com" << endl;
	QString ar = (artist.lower()).replace(" ", "_");
	QString al = (album.lower()).replace(" ", "_");
	QString l = ar.left(1);
	QString dcmime = "";
	KURL u1("http://img.darktown.com/cover/audio/download/"+l+"/"+ar+"_-_"+al+"_a.jpg");
	dcmime = KIO::NetAccess::mimetype( u1, NULL );
	if( dcmime == "image/jpeg" ){
		kdDebug("Tags") << "Cover found, downloading" << endl;
		//return KIO::NetAccess::download( u1, out, NULL );
	}
	else{
		KURL u2("http://img.darktown.com/cover/audio/download/"+l+"/"+ar+"_"+al+"_a.jpg");
		dcmime = KIO::NetAccess::mimetype( u2, NULL );
		if( dcmime == "image/jpeg" ){
			kdDebug("Tags") << "Cover found, downloading" << endl;
			//return KIO::NetAccess::download( u2, out, NULL );
		}
		else{
			QString ar = (upFirst(artist.lower())).replace(" ", "_");
			QString al = (upFirst(album.lower())).replace(" ", "_");
			KURL u3("http://img.darktown.com/cover/audio/download/"+l+"/"+ar+"_"+al+"_a.jpg");
			dcmime = KIO::NetAccess::mimetype( u3, NULL );
			if( dcmime == "image/jpeg" ){
				kdDebug("Tags") << "Cover found, downloading" << endl;
				//return KIO::NetAccess::download( u3, out, NULL );
			}
		}
	}
	kdDebug("Tags") << "Cover not found" << endl;
	return false;
}

QString Tags::upFirst( const QString & str){
	QStringList ls = 0;
	QString tmpstr = str;
	QString upstr = 0;
	tmpstr.replace("_"," ");
	ls = QStringList::split( ' ', tmpstr);
	QStringList::Iterator it = ls.begin();
	while( it!= ls.end() ){
		QString word = *it;
		QChar i = word.at(0);
		word.replace ( 0, 1, i.upper() );
		upstr += word + " " ;
		++it;
	}
	if(upstr.right(3) == " - "){
		upstr = upstr.remove(upstr.length() - 3, 3);
	}
	return upstr.stripWhiteSpace();
}

